<?php

namespace Drupal\popup_maker\Controller;

use Drupal\Core\Url;
use GuzzleHttp\Exception\RequestException;
use Zend\Diactoros\Response\RedirectResponse;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;

class SettingsController extends ControllerBase {

	/**
	 * The Messenger service.
	 *
	 * @var \Drupal\Core\Messenger\MessengerInterface
	 */
	protected $messenger;

	public function __construct(MessengerInterface $messenger)
	{
		$this->messenger = $messenger;
	}

	/**
	 * {@inheritdoc}
	 */
	public static function create(ContainerInterface $container) {
		return new static(
			$container->get('messenger')
		);
	}

	public function editPopupSettings()
	{
		$config = \Drupal::config('popup_maker.settings');

		$settings = [
			'api_key' => $config->get('api_key'),
			'popups' => $config->get('popups'),
			'user' => $config->get('user'),
			'popupSettings' => $config->get('popupSettings')
		];

		$contentTypes = \Drupal::service('entity.manager')->getStorage('node_type')->loadMultiple();

		$displayRules = [];
		if (!empty($contentTypes)) {
			foreach ($contentTypes as $key => $contentType) {
				$displayRules[$key] = [];
				$nodes = \Drupal::entityTypeManager()
					->getListBuilder('node')
					->getStorage()
					->loadByProperties(['type' => $key]);
				foreach ($nodes as $node) {
					$displayRules[$key][$node->id()] = $node->getTitle();
				}
			}
		}

		return array(
			'#theme' => 'popup_maker_settings',
			'#settings' => $settings,
			'#edit' => true,
			'#editingPopupId' => $_GET['id'],
			'#displayRules' => $displayRules,
			'#token' => \Drupal::csrfToken()->get('popup_maker_api_key'),
			'#attached' => array(
				'library' => array(
					'popup_maker/popup_maker_admin',
				),
			),
		);
	}

	public function show()
	{
		$config = \Drupal::config('popup_maker.settings');

		$settings = [
			'api_key' => $config->get('api_key'),
			'popups' => $config->get('popups'),
			'user' => $config->get('user'),
			'popupSettings' => $config->get('popupSettings')
		];

		return array(
			'#theme' => 'popup_maker_settings',
			'#settings' => $settings,
			'#token' => \Drupal::csrfToken()->get('popup_maker_api_key'),
			'#attached' => array(
				'library' => array(
					'popup_maker/popup_maker_admin',
				),
			),
		);
	}

	public function updatePopup()
	{
		$id = $_POST['id'];

		$config = \Drupal::service('config.factory')->getEditable('popup_maker.settings');

		$popupSettings = $config->get('popupSettings');

		$popupSettings[$id]['displayOptions'] = $_POST['sgpm_rules'];

		$config->set('popupSettings', $popupSettings)->save();

		return $this->redirect('popup_maker.settings.edit_popup_settings', ['id' => $id]);
	}

	public function refreshPopups()
	{
		$config = \Drupal::config('popup_maker.settings');

		if ($this->refreshData($config->get('api_key'))) {
			$this->messenger->addStatus('Popups list refreshed successfully');
		}

		return $this->redirect('popup_maker.settings');
	}

	public function updateApiKey()
	{
		if (!isset($_POST['sgpm-api-key-submit'])) {
			$this->messenger->addError('Invalid request');
			return false;
		}

		if (! \Drupal::csrfToken()->validate($_POST['_csrf_token'], 'popup_maker_api_key')) {
			$this->messenger->addError('Could not validate the security token');
			return false;
		}

		if ($this->refreshData($_POST['sgpm-api-key'])) {
			\Drupal::service('config.factory')
				->getEditable('popup_maker.settings')
				->set('api_key', $_POST['sgpm-api-key'])
				->save();

			$this->messenger->addStatus('New api key saved successfully');
		}

		return $this->redirect('popup_maker.settings');
	}

	public function updateStatus()
	{
		if (isset($_POST['sgpm-disable-popup'])) {
			$this->disablePopup();
		}

		if (isset($_POST['sgpm-enable-popup'])) {
			$this->enablePopup();
		}

		return $this->redirect('popup_maker.settings');

	}

	public function refreshData($apiKey)
	{
		if (empty($apiKey)) {
			$this->messenger->addError('Api Key is required to connect to the service');
			return false;
		}

		$data = array(
			'apiKey' => $apiKey,
			'appname' => 'Drupal'
		);

		$client = \Drupal::httpClient();

		try {
			$request = $client->post(SGPM_SERVICE_URL.'app/connect', [
				'form_params' => $data,
			]);
			$data = json_decode($request->getBody(),true);
		}
		catch (RequestException $e) {
			\Drupal::logger('popup_maker')->error($e);
			return false;
		}

		$config = \Drupal::service('config.factory')->getEditable('popup_maker.settings');

		if (!isset($data['isAuthenticate']) || !$data['isAuthenticate']) {
			$this->messenger->addError('Please, provide a valid Api Key');
			return false;
		}

		$config->set('popups', array_reverse($data['popups'], true))
			->set('user', $data['user'])
			->save();

		$popupSettings = $config->get('popupSettings');

		if (empty($popupSettings)) {
			$popupSettings = array();
		}

		foreach ($data['popups'] as $popupId => $popup) {
			if (!isset($popupSettings[$popupId])) {
				$popupSettings[$popupId] = array(
					'enabled' => 0,
					'displayOptions' => [],
				);
			}
		}

		$config->set('popupSettings', $popupSettings)->save();

		$this->messenger->addStatus('Data imported from service successfully');
		return true;
	}

	public function disablePopup()
	{
		if (!isset($_POST['id'])) {
			$this->messenger->addError('Parameter "id" is required to disable a popup');
			return false;
		}

		$id = abs(intval($_POST['id']));

		if ($id != $_POST['id']) {
			$this->messenger->addError('Parameter "id" must be non negative integer');
			return false;
		}

		$config = \Drupal::service('config.factory')->getEditable('popup_maker.settings');

		$popupSettings = $config->get('popupSettings');

		if (empty($popupSettings)) {
			$popupSettings = array();
		}

		if (!isset($popupSettings[$_POST['id']])) {
			$popupSettings[$_POST['id']] = array();
		}

		$popupSettings[$_POST['id']]['enabled'] = 0;

		$config->set('popupSettings', $popupSettings)->save();

		$this->messenger->addStatus('Popup Disabled successfully');
	}

	public function enablePopup()
	{
		if (!isset($_POST['id'])) {
			$this->messenger->addError('Parameter "id" is required to disable a popup');
			return false;
		}

		$id = abs(intval($_POST['id']));

		if ($id != $_POST['id']) {
			$this->messenger->addError('Parameter "id" must be non negative integer');
			return false;
		}

		$config = \Drupal::service('config.factory')->getEditable('popup_maker.settings');

		$popupSettings = $config->get('popupSettings');

		if (empty($popupSettings)) {
			$popupSettings = array();
		}

		if (!isset($popupSettings[$_POST['id']])) {
			$popupSettings[$_POST['id']] = array();
		}

		$popupSettings[$_POST['id']]['enabled'] = 1;

		$config->set('popupSettings', $popupSettings)->save();

		$this->messenger->addStatus('Popup Enabled successfully');
	}
}
